using System;
using System.Data;
using System.Data.SqlClient;
using DataAccess;
using System.EnterpriseServices;
using System.Runtime.CompilerServices;
using System.Reflection;

// Supply the COM+ application name. 
[assembly: ApplicationName("SalesComponent")]
// The strong-named assembly is supplied in the AssemblyInfo class.

namespace BusinessLogic
{
	/// <summary>
	/// Summary description for AutomaticSalesComponent.
	/// </summary>
	[Transaction(TransactionOption.Required)]
	public class AutomaticSalesComponent : ServicedComponent, ISalesComponent
	{
		public AutomaticSalesComponent()
		{
			//
			// TODO: Add constructor logic here
			//
		}
		public bool MakeSale(string connectionString, string customerID, int productID, int requestedQty, double unitPrice, double discount)
		{

			SqlConnection conn = new SqlConnection();
			bool successfulSale;
	        DataAccess.DataAccess dataAccess = new DataAccess.DataAccess();

			try 
			{

				conn.ConnectionString = connectionString;
				conn.Open();

				// attempt to add a new order. if this fails newOrder 
				// method will return false and transaction will be rolled back
				successfulSale = dataAccess.newOrder(conn, customerID, productID, requestedQty, unitPrice, discount);

				if (successfulSale) 
				{

					// attempt to adjust inventory, if this fails, adjustInventory 
					// method will return false and transaction will be rolled back
					successfulSale = dataAccess.adjustInventory(conn, requestedQty, productID);

					if (successfulSale)

						// check credit card limit. if over limit rollback transaction
						successfulSale = dataAccess.checkCreditCard();

				}

				if (successfulSale)

					// if the sale is successful, commit the transaction
					ContextUtil.SetComplete();

				else 
				{

					// if some part of the sale was unsuccessful (the credit card check), 
					// rollback the transaction
					ContextUtil.SetAbort();
					successfulSale = false;

				}

				return successfulSale;

			}  // End try

			catch (Exception ex)
			{

				// If an error occurs, rollback transaction
				ContextUtil.SetAbort();
				throw ex;

			}
			finally 
			{

				// if the connection is still open (possibly from an error) close the connection here.
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}
		}

	}
}
